"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Redis = void 0;
const redis_tag_cache_1 = __importDefault(require("redis-tag-cache"));
const Errors_1 = require("../Errors");
const Lambda_1 = require("../Lambda");
const autobind_decorator_1 = require("autobind-decorator");
const lodash_1 = require("lodash");
const MAX_AGE_IN_SECS = 6 * 60 * 60;
let redisTagCache = null;
function getRedisTagCache() {
    const port = Lambda_1.getEnvConfig('REDIS_PORT');
    const host = Lambda_1.getEnvConfig('REDIS_ENDPOINT');
    if (!redisTagCache && port && host && !isNaN(parseInt(port))) {
        redisTagCache = new redis_tag_cache_1.default({
            defaultTimeout: MAX_AGE_IN_SECS,
            redis: {
                port: parseInt(port),
                host: host,
            },
        });
    }
    return redisTagCache;
}
class Redis {
    constructor() {
        this.redis = null;
    }
    getOrCreateConnection() {
        if (!this.redis) {
            this.redis = getRedisTagCache();
            if (!this.redis)
                throw new Errors_1.InternalServerError('No redis cluster found');
        }
        return this.redis;
    }
    async get(key, options) {
        const connection = this.getOrCreateConnection();
        return new Promise((resolve, reject) => {
            var _a;
            const timeout = lodash_1.get(options, 'responseTimeoutInMillis') || 2000;
            const throwOnTimeout = (_a = lodash_1.get(options, 'throwOnTimeout')) !== null && _a !== void 0 ? _a : false;
            setTimeout(() => {
                if (!throwOnTimeout) {
                    resolve(null);
                    return;
                }
                reject(new Error('timeout'));
            }, timeout);
            connection.get(key).then(resolve).catch();
        });
    }
    async set(key, data, param3, param4) {
        const connection = this.getOrCreateConnection();
        return new Promise((resolve, reject) => {
            let tags = [];
            let ttlInSecs = undefined;
            let timeout = 2000;
            let throwOnTimeout = false;
            if (Array.isArray(param3)) {
                tags = param3;
                if (typeof param4 === 'object' && param4 !== null) {
                    ttlInSecs = lodash_1.get(param4, 'ttlInSecs') || undefined;
                    timeout = lodash_1.get(param4, 'responseTimeoutInMillis') || 2000;
                    throwOnTimeout = lodash_1.get(param4, 'throwOnTimeout') || false;
                }
            }
            else if (typeof param3 === 'object' && param3 !== null) {
                ttlInSecs = lodash_1.get(param3, 'ttlInSecs') || undefined;
                timeout = lodash_1.get(param3, 'responseTimeoutInMillis') || 2000;
                throwOnTimeout = lodash_1.get(param3, 'throwOnTimeout') || false;
            }
            setTimeout(() => {
                if (!throwOnTimeout) {
                    resolve();
                    return;
                }
                reject(new Error('timeout'));
            }, timeout);
            const options = ttlInSecs ? { timeout: ttlInSecs } : undefined;
            connection.set(key, data, tags, options).then(resolve).catch();
        });
    }
    async delete(key, options) {
        await this.set(key, null, options);
    }
    async invalidateTags(tags, options) {
        const connection = this.getOrCreateConnection();
        if (tags.length === 0)
            return;
        return new Promise((resolve, reject) => {
            var _a;
            const timeout = lodash_1.get(options, 'responseTimeoutInMillis') || 2000;
            const throwOnTimeout = (_a = lodash_1.get(options, 'throwOnTimeout')) !== null && _a !== void 0 ? _a : false;
            setTimeout(() => {
                if (!throwOnTimeout) {
                    resolve();
                    return;
                }
                reject(new Error('timeout'));
            }, timeout);
            connection
                .invalidate(...tags)
                .then(resolve)
                .catch();
        });
    }
}
__decorate([
    autobind_decorator_1.boundMethod
], Redis.prototype, "getOrCreateConnection", null);
__decorate([
    autobind_decorator_1.boundMethod
], Redis.prototype, "get", null);
__decorate([
    autobind_decorator_1.boundMethod
], Redis.prototype, "set", null);
__decorate([
    autobind_decorator_1.boundMethod
], Redis.prototype, "delete", null);
__decorate([
    autobind_decorator_1.boundMethod
], Redis.prototype, "invalidateTags", null);
exports.Redis = Redis;
